<?php
/*
parameters (version 1):
	output	type of output (json, html), optional (html is default)
			json to be implemented in future
	mode	specify visualization mode (table), optional, valid only for output=html

===
this script requires the following mysql table:

CREATE TABLE esp32iot (
			`date` DATETIME NOT NULL,
			`ipaddress` VARCHAR(16) NOT NULL,
			`key` VARCHAR(16) PRIMARY KEY,
			`value` VARCHAR(32) NOT NULL
)

===
IMPORTANT NOTICE, please read:

This software is licensed under the terms of the GNU GENERAL PUBLIC LICENSE,
please read the enclosed file license.txt or http://www.gnu.org/licenses/licenses.html

The use of this software is at the risk of the user.
*/
//define here mysql site specific info:
define("mysql_host", "localhost");
define("mysql_user", "your user");
define("mysql_pwd", "your password");
define("mysql_db", "your mysql db");

$output = @$_GET["output"];
$mode = @$_GET["mode"];
if (!isset($output))
	$output = "html";

if ($output == "html") {
	echo <<<EndHTML
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html lang="en">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
<meta http-equiv="refresh" content="10">
<title>ESP32 Demo</title>
<style type="text/css">
<!--
body       { background-color: #FFFFEE; color: #000000; font-family: Arial }
td.header  { background-color: #F0F000;}
.small       { font-size: 8pt; font-style: normal; font-family: Arial, Helvetica; }
table.db { background-color:#FFFFFF; border-width:3px; border-style:solid; border-color:#006699; }
td.db    { background-color: #DEE3E7; }
th.db    { background-color: #C8D1D7; }
-->
</style>
</head>
<body>
EndHTML;
}

if ($_SERVER['REQUEST_METHOD'] !== 'GET') {//this script shall be invoked with GET
	header("HTTP/1.1 405 Method Not Allowed");
	echo_error("invalid method, only GET is allowed");
	exit;
}

date_default_timezone_set("Europe/Rome");// added for PHP5 migration
mb_internal_encoding("UTF-8");

$link = connect_MySQL();
if ($link) {
	$query = "SELECT * FROM esp32iot";	
	do_query($link, $query, $output, $mode);
} else {
	header("HTTP/1.1 503 Service Unavailable");
	echo_error("cannot connect to MySQL");
}

////////////
// getRemoteAddr() returns the real IP address
// in qualche caso la variabile $_SERVER['REMOTE_ADDR'] restituisce l'indirizzo di un intermediario IP (es: CloudFlare) invece dell'indirizzo utente 
function getRemoteAddr(){
	  if (!empty($_SERVER['HTTP_X_FORWARDED_FOR']))
		return $_SERVER['HTTP_X_FORWARDED_FOR'];
	  else
		return $_SERVER['REMOTE_ADDR'];
}


//////////////
function connect_MySQL() {
	if (empty($_SESSION['mysql_error'])) {
		$link = @mysqli_connect(mysql_host, mysql_user, mysql_pwd);
		if ($link) {
			mysqli_select_db($link, mysql_db);
			if (!mysqli_errno($link))
				return $link;
			else mysqli_close($link);
		} else $_SESSION['mysql_error'] = true;
	}
}

//////////////
function do_query($link, $query, $output, $mode) {//perform query and print result
	$result = mysqli_query($link, $query);
	if ($result) {
		$counter = mysqli_num_rows($result);
		if ($counter == 0) {
			header("HTTP/1.1 404 Data not found");
		} else {
			if (($output != "html") || ($mode == "table"))
				print_result($link, $result, $output);	
			else {//display "temp" e "hum"
				$data = array();
				while($row=mysqli_fetch_assoc($result)) {
					$data[$row["key"]] = $row["value"];
				}
				echo "<table border=\"0\"><tr><th>Temperature</th><th>Humidity</th></tr>";
				echo "<tr>";
				echo "<td>"; svgwidget("vbar", 0, 40, $data["temp"], "°C", "#cc2311", 200, 200); echo "</td>";
				echo "<td>"; svgwidget("gauge", 0, 100, $data["hum"], "%", "#1123cc", 200, 400); echo "</td>";					
				echo "</tr></table>";
			}
		}
	} else {
		header("HTTP/1.1 404 Data not found");
		echo_error("Error MySQL: ".mysqli_error($link));
		exit;
	}
}

//////////////
function print_result($link, $result, $output) {
	if ($output == "json") {
		$data = array();
		while($row=mysqli_fetch_assoc($result)) {
			$data[$row["key"]] = $row["value"];
		}
		$json = "{";
		$i = 0;
		foreach($data as $key=>$value) {
			if ($i++ > 0)
				$json .= ", ";
			$json .= "\"".$key."\":".$value;
		}
		$json .= "}";
		header ("Content-type: application/json");
		echo $json;
	} else {//$output == "html", default
		$i = 0;
		print "<table class=\"db\"><tr>";
		while ($i < mysqli_field_count($link)) {
			$meta = mysqli_fetch_field_direct($result, $i);
			print "<th class=\"db\">".$meta->name."</th>";
			$i++;
		}
		print "</tr>";
		while($row=mysqli_fetch_assoc($result)) {
			print "<tr>";
			foreach($row as $key=>$value)
				print "<td class=\"db\">$value</td>";
			print "</tr>";
		}
		print "</table>";
	}
}

//////////////
/*svgwidget()
parameters:

widget:		type of widget: gauge, vbar (vertical bar), hbar (horizontal bar)
min:		min value
max:		max value
val:		actual value
unit:		measurement unit (optional)
fill:		fill color (optional)
height:		height of the image
width:		width of the image
*/
function svgwidget($widget, $min, $max, $val, $unit, $fill, $height, $width) {
	if (!isset($fill))
		$fill = "#202020";//dark gray as default filler
	if ($widget == "vbar") {//vertical bar
		$bheight = 90 * ($val - $min) / ($max - $min);//value from 0 to 70
		if ($bheight > 90) {
			$bheight = 90;
			$fill = "#ff0000";//show error with red bar
		}
		$bbase = 97 - $bheight;
		echo <<<EndHTML
	<svg svg width="{$width}" height="{$height}" viewBox="0 0 100 100">
		<rect x="10" y="7" width="20" height="90" fill="#ebebeb"></rect>
		<rect x="10" y="{$bbase}" width="20" height="{$bheight}" fill="{$fill}"></rect>
		<text x="37" y="54" text-anchor="start" style="font-weight: bold;font-size: 1em;">{$val} {$unit}</text>
		<text x="37" y="97" fill="#b3b3b3" style="font-size: 0.7em;">{$min}</text>
		<text x="37" y="11" text-anchor="start" fill="#b3b3b3" style="font-size: 0.7em;">{$max}</text>
	</svg>
EndHTML;
	} else if ($widget == "hbar") {//horizontal bar
		$bwidth = 70 * ($val - $min) / ($max - $min);//value from 0 to 70
		if ($bwidth > 70) {
			$bwidth = 70;
			$fill = "#ff0000";//show error with red bar
		}
		echo <<<EndHTML
	<svg svg width="{$width}" height="{$height}" viewBox="0 0 100 100">
		<rect x="17" y="21" width="70" height="20" fill="#ebebeb"></rect>
		<rect x="17" y="21" width="{$bwidth}" height="20" fill="{$fill}"></rect>
		<text x="57" y="17" text-anchor="middle" style="font-weight: bold;font-size: 1em;">{$val} {$unit}</text>
		<text x="87" y="17" text-anchor="middle" fill="#b3b3b3" style="font-size: 0.7em;">{$max}</text>
		<text x="17" y="17" text-anchor="middle" fill="#b3b3b3" style="font-size: 0.7em;">{$min}</text>
	</svg>
EndHTML;
	} else if ($widget == "gauge") {//gauge
		$degree = 180 * ($val - $min) / ($max - $min);//value from 0 to 180
		if ($degree > 180) {
			$degree = 180;
			$fill = "#ff0000";//show error with red bar
		}
		$rad = deg2rad($degree);
		$x_end = 100 - 90 * cos($rad);
		$y_end = 100 - 90 * sin($rad);
		echo <<<EndHTML
	<svg svg width="{$width}" height="{$height}" viewBox="0 0 200 100">
		<path d="M 10 100 A 90 90 0 0 1 190 100" stroke="#ebebeb" stroke-width="20" fill="none"></path>
		<path d="M 10 100 A 90 90 0 0 1 {$x_end} {$y_end}" stroke="{$fill}" stroke-width="12" fill="none"></path>
		<text x="100" y="80" text-anchor="middle" alignment-baseline="middle" style="font-weight: bold;font-size: 1em;">{$val} {$unit}</text>
	</svg>
EndHTML;
	/*
	<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 200 100">
		<path d="M 190 100 A 90 90 0 0 0 10 100" stroke="{$fill}" stroke-width="12" fill="none"></path>
		<text x="100" y="80" text-anchor="middle" alignment-baseline="middle" font-size="28" font-weight="bold" fill="{$fill}">{$val} {$unit}</text>
	</svg>

	*/
	}
}

function echo_error($msg, $trigger = true) {
	echo $msg;
	if ($trigger)
		trigger_error($msg);
}


if ($output == "html") {
	echo <<<EndHTML
</body>
</html>
EndHTML;
}
?>